@@ -6,12 +6,15 @@ import logging
 from requests.models import Response
 from localstack import config
 from localstack.services import plugins
+from localstack.dashboard import infra as dashboard_infra
 from localstack.constants import (
     HEADER_LOCALSTACK_TARGET, HEADER_LOCALSTACK_EDGE_URL, LOCALSTACK_ROOT_FOLDER, PATH_USER_REQUEST)
 from localstack.utils.common import run, is_root, TMP_THREADS, to_bytes, truncate, to_str, get_service_protocol
 from localstack.utils.common import safe_requests as requests
+from localstack.utils.aws.aws_stack import Environment
 from localstack.services.generic_proxy import ProxyListener, start_proxy_server
 from localstack.services.sqs.sqs_listener import is_sqs_queue_url
+from localstack.utils.aws.aws_stack import set_default_region_in_headers
 
 LOG = logging.getLogger(__name__)
 
@@ -28,6 +31,8 @@ class ProxyListenerEdge(ProxyListener):
 
         if path.split('?')[0] == '/health':
             return serve_health_endpoint(method, path, data)
+        if method == 'POST' and path == '/graph':
+            return serve_resource_graph(data)
 
         # kill the process if we receive this header
         headers.get(HEADER_KILL_SIGNAL) and os._exit(0)
@@ -40,6 +45,8 @@ class ProxyListenerEdge(ProxyListener):
         # extract API details
         api, port, path, host = get_api_from_headers(headers, path)
 
+        set_default_region_in_headers(headers)
+
         if port and int(port) < 0:
             return 404
 
@@ -61,13 +68,18 @@ class ProxyListenerEdge(ProxyListener):
 
         connect_host = '%s:%s' % (config.HOSTNAME, port)
         url = '%s://%s%s' % (get_service_protocol(), connect_host, path)
+
         headers['Host'] = host
-        function = getattr(requests, method.lower())
         if isinstance(data, dict):
             data = json.dumps(data)
 
-        response = function(url, data=data, headers=headers, verify=False, stream=True)
-        return response
+        return do_forward_request(api, method, url, data, headers)
+
+
+def do_forward_request(api, method, url, data, headers):
+    function = getattr(requests, method.lower())
+    response = function(url, data=data, headers=headers, verify=False, stream=True)
+    return response
 
 
 def get_api_from_headers(headers, path=None):
@@ -113,7 +125,7 @@ def get_api_from_headers(headers, path=None):
     elif target.startswith('DynamoDBStreams') or host.startswith('streams.dynamodb.'):
         # Note: DDB streams requests use ../dynamodb/.. auth header, hence we also need to update result_before
         result = result_before = 'dynamodbstreams', config.PORT_DYNAMODBSTREAMS
-    elif ls_target == 'web' or path == '/graph':
+    elif ls_target == 'web':
         result = 'web', config.PORT_WEB_UI
 
     return result[0], result_before[1] or result[1], path, host
@@ -137,6 +149,13 @@ def serve_health_endpoint(method, path, data):
         return {'status': 'OK'}
 
 
+def serve_resource_graph(data):
+    data = json.loads(to_str(data or '{}'))
+    env = Environment.from_string(data.get('awsEnvironment'))
+    graph = dashboard_infra.get_graph(name_filter=data.get('nameFilter') or '.*', env=env)
+    return graph
+
+
 def get_port_from_custom_rules(method, path, data, headers):
     """ Determine backend port based on custom rules. """
 
@@ -169,6 +188,9 @@ def get_port_from_custom_rules(method, path, data, headers):
 
     # detect S3 URLs
     if stripped and '/' not in stripped:
+        if method == 'HEAD':
+            # assume that this is an S3 HEAD bucket request with URL path `/<bucket>`
+            return config.PORT_S3
         if method == 'PUT':
             # assume that this is an S3 PUT bucket request with URL path `/<bucket>`
             return config.PORT_S3
@@ -176,6 +198,10 @@ def get_port_from_custom_rules(method, path, data, headers):
             # assume that this is an S3 POST request with form parameters or multipart form in the body
             return config.PORT_S3
 
+    if stripped.count('/') == 1 and method == 'PUT':
+        # assume that this is an S3 PUT bucket object request with URL path `/<bucket>/object`
+        return config.PORT_S3
+
     # detect S3 requests sent from aws-cli using --no-sign-request option
     if 'aws-cli/' in headers.get('User-Agent', ''):
         return config.PORT_S3
