@@ -4,7 +4,7 @@ import logging
 import traceback
 import requests
 from localstack import config
-from localstack.utils.common import get_service_protocol
+from localstack.utils.common import get_service_protocol, clone
 from localstack.utils.bootstrap import canonicalize_api_names
 
 # set up logger
@@ -70,7 +70,9 @@ def register_plugin(plugin):
 def get_services_health(reload=False):
     if reload:
         reload_services_health()
-    return dict(STATUSES)
+    result = clone(dict(STATUSES))
+    result.get('services', {}).pop('edge', None)
+    return result
 
 
 def set_services_health(data):
@@ -121,9 +123,9 @@ def record_service_health(api, status):
     data = {
         api: status
     }
-    health_url = '%s://%s:%s/health' % (get_service_protocol(), config.LOCALHOST, config.PORT_WEB_UI)
+    health_url = '%s://%s:%s/health' % (get_service_protocol(), config.LOCALHOST, config.EDGE_PORT)
     try:
-        requests.put(health_url, data=json.dumps(data))
+        requests.put(health_url, data=json.dumps(data), verify=False)
     except Exception:
         # ignore for now, if the service is not running
         pass
