@@ -1,3 +1,4 @@
+import gzip
 import re
 import json
 import uuid
@@ -9,6 +10,7 @@ import datetime
 import xmltodict
 import collections
 import dateutil.parser
+import urllib.parse
 import six
 import botocore.config
 from pytz import timezone
@@ -154,7 +156,7 @@ def get_event_message(event_name, bucket_name, file_name='testfile.txt', etag=''
                     'arn': 'arn:aws:s3:::%s' % bucket_name
                 },
                 'object': {
-                    'key': file_name,
+                    'key': urllib.parse.quote(file_name),
                     'size': file_size,
                     'eTag': etag,
                     'versionId': version_id,
@@ -500,6 +502,17 @@ def fix_creation_date(method, path, response):
         r'\1Z</CreationDate>', to_str(response._content))
 
 
+def fix_delimiter(data, headers, response):
+    if response.status_code == 200 and response._content:
+        c, xml_prefix, delimiter = response._content, '<?xml', '<Delimiter><'
+        pattern = '[<]Delimiter[>]None[<]'
+        if isinstance(c, bytes):
+            xml_prefix, delimiter = xml_prefix.encode(), delimiter.encode()
+            pattern = pattern.encode()
+        if c.startswith(xml_prefix):
+            response._content = re.compile(pattern).sub(delimiter, c)
+
+
 def convert_to_chunked_encoding(method, path, response):
     if method != 'GET' or path != '/':
         return
@@ -510,6 +523,21 @@ def convert_to_chunked_encoding(method, path, response):
     response.headers.pop('Content-Length', None)
 
 
+def unquote(s):
+    if (s[0], s[-1]) in (('"', '"'), ("'", "'")):
+        return s[1:-1]
+    return s
+
+
+def ret304_on_etag(data, headers, response):
+    etag = response.headers.get('ETag')
+    if etag:
+        match = headers.get('If-None-Match')
+        if match and unquote(match) == unquote(etag):
+            response.status_code = 304
+            response._content = ''
+
+
 def fix_etag_for_multipart(data, headers, response):
     # Fix for https://github.com/localstack/localstack/issues/1978
     if headers.get(CONTENT_SHA256_HEADER) == STREAMING_HMAC_PAYLOAD:
@@ -1198,7 +1226,9 @@ class ProxyListenerS3(PersistingProxyListener):
             fix_metadata_key_underscores(response=response)
             fix_creation_date(method, path, response=response)
             fix_etag_for_multipart(data, headers, response)
+            ret304_on_etag(data, headers, response)
             append_aws_request_troubleshooting_headers(response)
+            fix_delimiter(data, headers, response)
 
             if method == 'PUT':
                 set_object_expiry(path, headers)
@@ -1269,6 +1299,11 @@ class ProxyListenerS3(PersistingProxyListener):
             # convert to chunked encoding, for compatibility with certain SDKs (e.g., AWS PHP SDK)
             convert_to_chunked_encoding(method, path, response)
 
+            if headers.get('Accept-Encoding') == 'gzip':
+                response._content = gzip.compress(to_bytes(response._content))
+                response.headers['Content-Length'] = str(len(response._content))
+                response.headers['Content-Encoding'] = 'gzip'
+
 
 # instantiate listener
 UPDATE_S3 = ProxyListenerS3()
