@@ -1,8 +1,8 @@
-from __future__ import print_function
-
 import time
 import json
 import six
+from datetime import datetime
+from localstack.utils.common import isoformat_milliseconds
 
 if six.PY3:
     long = int
@@ -25,11 +25,12 @@ class Component(object):
 
 
 class KinesisStream(Component):
-    def __init__(self, id, params={}, num_shards=1, connection=None):
+    def __init__(self, id, params=None, num_shards=1, connection=None):
         super(KinesisStream, self).__init__(id)
+        params = params or {}
         self.shards = []
-        self.stream_name = params['name'] if 'name' in params else self.name()
-        self.num_shards = params['shards'] if 'shards' in params else num_shards
+        self.stream_name = params.get('name', self.name())
+        self.num_shards = params.get('shards', num_shards)
         self.conn = connection
         self.stream_info = params
 
@@ -61,19 +62,21 @@ class KinesisStream(Component):
         return self.conn.put_record(StreamName=self.stream_name, Data=data, PartitionKey=key)
 
     def read(self, amount=-1, shard='shardId-000000000001'):
-        s_iterator = kinesis_conn.get_shard_iterator(self.stream_name, shard, 'TRIM_HORIZON')
-        record = kinesis_conn.get_records(s_iterator['ShardIterator'])
+        if not self.conn:
+            raise Exception('Please create the Kinesis connection first.')
+        s_iterator = self.conn.get_shard_iterator(self.stream_name, shard, 'TRIM_HORIZON')
+        record = self.conn.get_records(s_iterator['ShardIterator'])
         while True:
             try:
                 if record['NextShardIterator'] is None:
                     break
                 else:
-                    next_entry = kinesis_conn.get_records(record['NextShardIterator'])
+                    next_entry = self.conn.get_records(record['NextShardIterator'])
                     if len(next_entry['Records']):
                         print(next_entry['Records'][0]['Data'])
                     record = next_entry
             except Exception as e:
-                print('Error reading from Kinesis stream "%s": %s' (self.stream_name, e))
+                print('Error reading from Kinesis stream "%s": %s' % (self.stream_name, e))
 
     def wait_for(self):
         GET_STATUS_SLEEP_SECS = 5
@@ -83,7 +86,7 @@ class KinesisStream(Component):
                 status = self.get_status()
                 if status == 'ACTIVE':
                     return
-            except Exception as e:
+            except Exception:
                 # swallowing this exception should be ok, as we are in a retry loop
                 pass
             time.sleep(GET_STATUS_SLEEP_SECS)
@@ -95,12 +98,12 @@ class KinesisStream(Component):
 
 
 class KinesisShard(Component):
-    MAX_KEY = "340282366920938463463374607431768211455"
+    MAX_KEY = '340282366920938463463374607431768211455'
 
     def __init__(self, id):
         super(KinesisShard, self).__init__(id)
         self.stream = None
-        self.start_key = "0"
+        self.start_key = '0'
         self.end_key = KinesisShard.MAX_KEY  # 128 times '1' binary as decimal
         self.child_shards = []
 
@@ -154,13 +157,143 @@ class FirehoseStream(KinesisStream):
 
 
 class LambdaFunction(Component):
-    def __init__(self, id):
-        super(LambdaFunction, self).__init__(id)
+    def __init__(self, arn):
+        super(LambdaFunction, self).__init__(arn)
         self.event_sources = []
         self.targets = []
+        self.versions = {}
+        self.aliases = {}
+        self.envvars = {}
+        self.tags = {}
+        self.concurrency = None
+        self.runtime = None
+        self.handler = None
+        self.cwd = None
+        self.timeout = None
+        self.last_modified = None
+        self.vpc_config = None
+        self.role = None
+        self.kms_key_arn = None
+        self.memory_size = None
+        self.code = None
+        self.dead_letter_config = None
+        self.on_successful_invocation = None
+        self.max_retry_attempts = None
+        self.max_event_age = None
+        self.description = ''
+
+    def set_dead_letter_config(self, data):
+        config = data.get('DeadLetterConfig')
+        if not config:
+            return
+        self.dead_letter_config = config
+        target_arn = config.get('TargetArn') or ''
+        if ':sqs:' not in target_arn and ':sns:' not in target_arn:
+            raise Exception('Dead letter queue ARN "%s" requires a valid SQS queue or SNS topic' % target_arn)
+
+    def get_function_event_invoke_config(self):
+        response = {
+            'LastModified': str(self.last_modified),
+            'FunctionArn': str(self.id),
+        }
+
+        if self.max_retry_attempts:
+            response.update({'MaximumRetryAttempts': self.max_retry_attempts})
+
+        if self.max_event_age:
+            response.update({'MaximumEventAgeInSeconds': self.max_event_age})
+
+        if self.on_successful_invocation or self.dead_letter_config:
+            response.update({'DestinationConfig': {}})
+            if self.on_successful_invocation:
+                response['DestinationConfig'].update({
+                    'OnSuccess': {
+                        'Destination': self.on_successful_invocation
+                    }
+                })
+            if self.dead_letter_config:
+                response['DestinationConfig'].update({
+                    'OnFailure': {
+                        'Destination': self.dead_letter_config
+                    }
+                })
+
+        return response
+
+    def clear_function_event_invoke_config(self):
+        if hasattr(self, 'dead_letter_config'):
+            self.dead_letter_config = None
+        if hasattr(self, 'on_successful_invocation'):
+            self.on_successful_invocation = None
+        if hasattr(self, 'max_retry_attempts'):
+            self.max_retry_attempts = None
+        if hasattr(self, 'max_event_age'):
+            self.max_event_age = None
+
+    def put_function_event_invoke_config(self, data):
+        if not isinstance(data, dict):
+            return
+
+        updated = False
+        if 'DestinationConfig' in data:
+            if 'OnFailure' in data['DestinationConfig']:
+                dlq_arn = data['DestinationConfig']['OnFailure']['Destination']
+                self.dead_letter_config = dlq_arn
+                updated = True
+
+            if 'OnSuccess' in data['DestinationConfig']:
+                sq_arn = data['DestinationConfig']['OnSuccess']['Destination']
+                self.on_successful_invocation = sq_arn
+                updated = True
+
+        if 'MaximumRetryAttempts' in data:
+            try:
+                max_retry_attempts = int(data['MaximumRetryAttempts'])
+            except Exception:
+                max_retry_attempts = 3
+
+            self.max_retry_attempts = max_retry_attempts
+            updated = True
+
+        if 'MaximumEventAgeInSeconds' in data:
+            try:
+                max_event_age = int(data['MaximumEventAgeInSeconds'])
+            except Exception:
+                max_event_age = 3600
+
+            self.max_event_age = max_event_age
+            updated = True
+
+        if updated:
+            self.last_modified = isoformat_milliseconds(datetime.utcnow()) + '+0000'
+
+        return self
+
+    def get_version(self, version):
+        return self.versions.get(version)
+
+    def max_version(self):
+        versions = [int(key) for key in self.versions.keys() if key != '$LATEST']
+        return versions and max(versions) or 0
 
     def name(self):
-        return self.id.split(':function:')[-1]
+        # Example ARN: arn:aws:lambda:aws-region:acct-id:function:helloworld:1
+        return self.id.split(':')[6]
+
+    def arn(self):
+        return self.id
+
+    def function(self, qualifier=None):
+        return self.versions.get(self.get_qualifier_version(qualifier)).get('Function')
+
+    def get_qualifier_version(self, qualifier=None):
+        if not qualifier:
+            qualifier = '$LATEST'
+        return qualifier if qualifier in self.versions else \
+            self.aliases.get(qualifier).get('FunctionVersion')
+
+    def qualifier_exists(self, qualifier):
+        return qualifier in self.aliases or qualifier in self.versions
 
     def __str__(self):
         return '<%s:%s>' % (self.__class__.__name__, self.name())
@@ -238,7 +371,8 @@ class EventSource(Component):
         super(EventSource, self).__init__(id)
 
     @staticmethod
-    def get(obj, pool={}, type=None):
+    def get(obj, pool=None, type=None):
+        pool = pool or {}
         if not obj:
             return None
         if isinstance(obj, Component):
@@ -248,7 +382,7 @@ class EventSource(Component):
         inst = None
         if obj.startswith('arn:aws:kinesis:'):
             inst = KinesisStream(obj)
-        if obj.startswith('arn:aws:lambda:'):
+        elif obj.startswith('arn:aws:lambda:'):
             inst = LambdaFunction(obj)
         elif obj.startswith('arn:aws:dynamodb:'):
             if '/stream/' in obj:
@@ -258,6 +392,8 @@ class EventSource(Component):
                 inst.table = table
             else:
                 inst = DynamoDB(obj)
+        elif obj.startswith('arn:aws:sqs:'):
+            inst = SqsQueue(obj)
         elif type:
             for o in EventSource.filter_type(pool, type):
                 if o.name() == obj:
@@ -266,13 +402,9 @@ class EventSource(Component):
                     if o.endpoint == obj:
                         return o
         else:
-            print("Unexpected object name! %s" % obj)
+            print("Unexpected object name: '%s'" % obj)
         return inst
 
     @staticmethod
     def filter_type(pool, type):
-        result = []
-        for key, obj in six.iteritems(pool):
-            if isinstance(obj, type):
-                result.append(obj)
-        return result
+        return [obj for obj in six.itervalues(pool) if isinstance(obj, type)]
