@@ -13,6 +13,7 @@ import six
 import shutil
 import pip as pip_mod
 from datetime import datetime
+from concurrent.futures._base import Future
 from localstack import constants, config
 
 # set up logger
@@ -44,7 +45,8 @@ PLUGIN_SCOPE_COMMANDS = 'commands'
 API_DEPENDENCIES = {
     'dynamodbstreams': ['kinesis'],
     'es': ['elasticsearch'],
-    'lambda': ['logs']
+    'lambda': ['logs'],
+    'kinesis': ['dynamodb']
 }
 # composites define an abstract name like "serverless" that maps to a set of services
 API_COMPOSITES = {
@@ -52,9 +54,6 @@ API_COMPOSITES = {
     'cognito': ['cognito-idp', 'cognito-identity']
 }
 
-# name of main Docker container
-MAIN_CONTAINER_NAME = 'localstack_main'
-
 # environment variable that indicates that we're executing in
 # the context of the script that starts the Docker container
 ENV_SCRIPT_STARTING_DOCKER = 'LS_SCRIPT_STARTING_DOCKER'
@@ -174,7 +173,7 @@ def docker_container_running(container_name):
 
 
 def get_docker_container_names():
-    cmd = "docker ps --format '{{.Names}}'"
+    cmd = "%s ps --format '{{.Names}}'" % config.DOCKER_CMD
     try:
         output = to_str(run(cmd))
         container_names = re.split(r'\s+', output.strip().replace('\n', ' '))
@@ -185,12 +184,22 @@ def get_docker_container_names():
 
 
 def get_main_container_ip():
-    cmd = "docker inspect -f '{{range .NetworkSettings.Networks}}{{.IPAddress}}{{end}}' %s" % MAIN_CONTAINER_NAME
+    container_name = get_main_container_name()
+    cmd = ("%s inspect -f '{{range .NetworkSettings.Networks}}{{.IPAddress}}{{end}}' %s" %
+        (config.DOCKER_CMD, container_name))
     return run(cmd).strip()
 
 
+def get_main_container_name():
+    cmd = "%s inspect -f '{{ .Name }}' %s" % (config.DOCKER_CMD, config.HOSTNAME)
+    return run(cmd).strip().lstrip('/')
+
+
 def setup_logging():
     # determine and set log level
+    if PLUGINS_LOADED.get('_logging_'):
+        return
+    PLUGINS_LOADED['_logging_'] = True
     log_level = logging.DEBUG if is_debug() else logging.INFO
     logging.basicConfig(level=log_level, format=LOG_FORMAT, datefmt=LOG_DATE_FORMAT)
 
@@ -300,6 +309,8 @@ class PortMappings(object):
             return
         if int(port) <= 0:
             raise Exception('Unable to add mapping for invalid port: %s' % port)
+        if self.contains(port):
+            return
         for from_range, to_range in self.mappings.items():
             if not self.in_expanded_range(port, from_range):
                 continue
@@ -318,6 +329,11 @@ class PortMappings(object):
 
         return ' '.join([entry(k, v) for k, v in self.mappings.items()])
 
+    def contains(self, port):
+        for from_range, to_range in self.mappings.items():
+            if self.in_range(port, from_range):
+                return True
+
     def in_range(self, port, range):
         return port >= range[0] and port <= range[1]
 
@@ -361,7 +377,7 @@ def extract_port_flags(user_flags, port_mappings):
 
 def start_infra_in_docker():
 
-    container_name = MAIN_CONTAINER_NAME
+    container_name = config.MAIN_CONTAINER_NAME
 
     if docker_container_running(container_name):
         raise Exception('LocalStack container named "%s" is already running' % container_name)
@@ -374,7 +390,6 @@ def start_infra_in_docker():
     # prepare APIs
     canonicalize_api_names()
 
-    services = os.environ.get('SERVICES', '')
     entrypoint = os.environ.get('ENTRYPOINT', '')
     cmd = os.environ.get('CMD', '')
     user_flags = config.DOCKER_FLAGS
@@ -400,11 +415,6 @@ def start_infra_in_docker():
     for port in service_ports.values():
         port_mappings.add(port)
 
-    if services:
-        port_mappings = PortMappings()
-        for port in set(service_ports.values()):
-            port_mappings.add(port)
-
     env_str = ''
     for env_var in config.CONFIG_ENV_VARS:
         value = os.environ.get(env_var, None)
@@ -503,23 +513,30 @@ class FuncThread(threading.Thread):
         self.params = params
         self.func = func
         self.quiet = quiet
+        self.result_future = Future()
 
     def run(self):
+        result = None
         try:
-            self.func(self.params)
-        except Exception:
+            result = self.func(self.params)
+        except Exception as e:
             if not self.quiet:
-                LOG.warning('Thread run method %s(%s) failed: %s' %
-                    (self.func, self.params, traceback.format_exc()))
+                LOG.warning('Thread run method %s(%s) failed: %s %s' %
+                    (self.func, self.params, e, traceback.format_exc()))
+        finally:
+            try:
+                self.result_future.set_result(result)
+            except Exception:
+                # this can happen as InvalidStateError on shutdown, if the task is already canceled
+                pass
 
     def stop(self, quiet=False):
         if not quiet and not self.quiet:
             LOG.warning('Not implemented: FuncThread.stop(..)')
 
 
-def run(cmd, print_error=True, asynchronous=False, stdin=False,
-        stderr=subprocess.STDOUT, outfile=None, env_vars=None, inherit_cwd=False,
-        inherit_env=True, tty=False):
+def run(cmd, print_error=True, asynchronous=False, stdin=False, stderr=subprocess.STDOUT,
+        outfile=None, env_vars=None, inherit_cwd=False, inherit_env=True, tty=False):
     env_dict = os.environ.copy() if inherit_env else {}
     if env_vars:
         env_dict.update(env_vars)
@@ -579,19 +596,18 @@ def run(cmd, print_error=True, asynchronous=False, stdin=False,
 
 
 def is_mac_os():
-    try:
-        out = to_str(subprocess.check_output('uname -a', shell=True))
-        return 'Darwin' in out
-    except subprocess.CalledProcessError:
-        return False
+    return 'Darwin' in get_uname()
 
 
 def is_linux():
+    return 'Linux' in get_uname()
+
+
+def get_uname():
     try:
-        out = to_str(subprocess.check_output('uname -a', shell=True))
-        return 'Linux' in out
-    except subprocess.CalledProcessError:
-        return False
+        return to_str(subprocess.check_output('uname -a', shell=True))
+    except Exception:
+        return ''
 
 
 def mkdir(folder):
