@@ -30,7 +30,7 @@ from contextlib import closing
 from datetime import datetime, date
 from six import with_metaclass
 from six.moves import cStringIO as StringIO
-from six.moves.urllib.parse import urlparse
+from six.moves.urllib.parse import urlparse, parse_qs
 from multiprocessing.dummy import Pool
 from localstack import config
 from localstack.config import DEFAULT_ENCODING
@@ -71,6 +71,11 @@ CACHE = {}
 SSL_CERT_LOCK = threading.RLock()
 
 
+class Mock(object):
+    """ Dummy class that can be used for mocking custom attributes. """
+    pass
+
+
 class CustomEncoder(json.JSONEncoder):
     """ Helper class to convert JSON documents with datetime, decimals, or bytes. """
 
@@ -81,7 +86,7 @@ class CustomEncoder(json.JSONEncoder):
             else:
                 return int(o)
         if isinstance(o, (datetime, date)):
-            return str(o)
+            return timestamp_millis(o)
         if isinstance(o, six.binary_type):
             return to_str(o)
         try:
@@ -306,12 +311,18 @@ class CaptureOutput(object):
 # ----------------
 
 def start_thread(method, *args, **kwargs):
+    _shutdown_hook = kwargs.pop('_shutdown_hook', True)
     thread = FuncThread(method, *args, **kwargs)
     thread.start()
-    TMP_THREADS.append(thread)
+    if _shutdown_hook:
+        TMP_THREADS.append(thread)
     return thread
 
 
+def start_worker_thread(method, *args, **kwargs):
+    return start_thread(method, *args, _shutdown_hook=False, **kwargs)
+
+
 def synchronized(lock=None):
     """
     Synchronization decorator as described in
@@ -663,6 +674,19 @@ def download(url, path, verify_ssl=True):
         s.close()
 
 
+def parse_request_data(method, path, data):
+    """ Extract request data either from query string (for GET) or request body (for POST). """
+    if method == 'POST':
+        result = parse_qs(to_str(data))
+    elif method == 'GET':
+        parsed_path = urlparse(path)
+        result = parse_qs(parsed_path.query)
+    else:
+        return {}
+    result = dict([(k, v[0]) for k, v in result.items()])
+    return result
+
+
 def first_char_to_lower(s):
     return '%s%s' % (s[0].lower(), s[1:])
 
